/*
DarkEnergy

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU.

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


/**
 * Algorithme de simulation pour SimuApplet.
 * Ce programme calcule l'age de l'univers et l'évolution du facteur d'echelle "a" en fonction du temps "t"
 * Les modeles d'Univers consideres sont ceux de Friedmann-Lemaitre-Robertson-Walker (AgeUnivers)
 */

new DarkEnergy();

function DarkEnergy() {
    this.applet = new SimuApplet(this);
    this.applet.appletDoc("dark_energy_doc.xml");
    
	this.mpc = 3.08567758074e22;
	this.gyr = 3.1688087814e-17;
	this.c = 2.99792458e8;
	this.twopi = 6.283185307179586;
	this.a0 = 1.0;
	this.h0 = 0.;
    this.Omega_m0 = 0.;
    this.Omega_rad0 = 0.;
    this.Omega_l0 = 0.;
    this.Omega_k0 = 0.;
	this.fsn1a = '';
	this.stop = 1e-6;
	this.dz = 1e-3;
	this.idat = 0;
	this.sum_int = 0.;
    this.a = 0.;
    this.t = 0.;
    this.h = 0.;
    this.integral = 0.;
    this.age = 0.;
    this.courbure = 0.;
    this.muz = 0.;
    this.chi2 = 0.;
    this.dl = 0.;
    this.z = 0.;
    this.sum1 = 0.;
    this.sum2 = 0.;
    this.sum3 = 0.;
    this.tempmuz = 0.;
    this.yz = 0.;
	this.zobs = null; // double[]
    this.muobs = null; // double[]
    this.sigobs = null; // double[]
	this.len = 0;
	this.xMin = +0.015;
    this.xMax = +1.01;// abcisses
	this.yMin = +34.071;
    this.yMax = +44.673;// ordonnées
}

DarkEnergy.prototype.initCalculLive = function(lin) {
    var data;
    
    this.h0 = lin.lireDouble('h0');
    this.Omega_m0 = lin.lireDouble('omega-m0');
    this.Omega_l0 = lin.lireDouble('omega-l0');
    this.fsn1a = lin.lireString('fsn1a');
    this.Omega_rad0 = lin.lireDouble('omega-rad0');
    this.Omega_rad0 = 4.1534e-5 / this.h0 / this.h0 * 100 * 100 * Math.pow((this.Omega_rad0 / 2.725), 4);
    
    this.z = 0;
    this.dl = 0;
    this.muz = 0;
    this.tempmuz = 0;
    this.chi2 = 0;
    this.sum1 = 0;
    this.sum2 = 0;
    this.sum3 = 0;
    this.sum_int = 0;
    this.idat = 0;
    
    this.h0 = this.h0 * 1.e3 / this.mpc;
    this.Omega_k0 = this.Omega_rad0 + this.Omega_m0 + this.Omega_l0 - 1.;
    
    data = this.lireFichier(this.fsn1a);
    if (data == null)
        return;
    this.zobs = data[0];
    this.muobs = data[1];
    this.sigobs = data[2];
    this.len = this.zobs.length;
}

// Calcul de zobs(j) à zobs(j+1)
// Attention les données doivent être ordonnées par ordre croissant de redshift
DarkEnergy.prototype.calculLive = function() {
    var out;
    
    if (this.z >= this.zobs[this.len - 1]){
        return null; // fin du calcul
    }
    out = new ListeValeurs();
    this.integral = this.rombint(this.f2, this.z, this.zobs[this.idat], this.stop);
    this.z = this.zobs[this.idat];
    this.sum_int += this.integral;

    if (this.Omega_k0 > 0) { // elliptic geometry
        this.dl = (1. + this.z) / Math.sqrt(Math.abs(this.Omega_k0))
            * Math.sin(this.sum_int * Math.sqrt(Math.abs(this.Omega_k0)))
            * 3.e3 / this.h0 * 1e5 / this.mpc;
    }
    if (this.Omega_k0 < 0) { // hyperbolic geometry
        this.dl = this.sum_int * Math.sqrt(Math.abs(this.Omega_k0));
        this.dl = (1. + this.z) / Math.sqrt(Math.abs(this.Omega_k0))
            * (Math.exp(this.dl)-Math.exp(-this.dl))
            / 2 * 3.e3 / this.h0 * 1e5 / this.mpc;
    } else {
        this.dl = this.sum_int * (this.z + 1.) * 3.e3 / this.h0 * 1e5 / this.mpc;
    }

    if (this.fsn1a == 'Daly03.dat') {
        this.muz = this.dl / (this.z + 1.) / (3.e3 / this.h0 * 1e5 / this.mpc);
    } else {
        this.muz = 25. + 5. * Math.log(this.dl) / Math.log(10.);
    }

    if (this.fsn1a == 'SNLS06.dat') {
        this.sum1 += (this.muobs[this.idat] - this.muz)
            * (this.muobs[this.idat] - this.muz)
            / (this.sigobs[this.idat] * this.sigobs[this.idat] + 0.13 * 0.13);
        this.sum2 += (this.muobs[this.idat] - this.muz) / (this.sigobs[this.idat] * this.sigobs[this.idat] + 0.13 * 0.13);
        this.sum3 += 1. / (this.sigobs[this.idat] * this.sigobs[this.idat] + 0.13 * 0.13);
    }
    if (this.fsn1a == 'Riess04.dat') {
        this.sum1 += (this.muobs[this.idat] - this.muz)
            * (this.muobs[this.idat] - this.muz)
            / (this.sigobs[this.idat] * this.sigobs[this.idat]);
        this.sum2 += (this.muobs[this.idat] - this.muz) / (this.sigobs[this.idat] * this.sigobs[this.idat]);
        this.sum3 += 1. / (this.sigobs[this.idat] * this.sigobs[this.idat]);
    }
    if (this.fsn1a == 'Daly03.dat' ){
        this.sum1 += (this.muobs[this.idat] - this.muz)
            * (this.muobs[this.idat] - this.muz)
            / (this.sigobs[this.idat] * this.sigobs[this.idat]);
    }
    
    out.ajouterDouble('muz', this.muz);
    out.ajouterDouble('z', this.z);
    out.ajouterDouble('muzobs', this.muobs[this.idat]);
    out.ajouterDouble('zobs', this.zobs[this.idat]);
    this.idat++;
    if (this.fsn1a == 'Daly03.dat') {
        this.chi2 = this.sum1;
        out.ajouterString("comment", "chi^2 = " + (Math.floor(10. * this.chi2) / 10.) + " pour " + this.len + " données");
    } else {
        this.chi2 = this.sum1 - this.sum2 * this.sum2 / this.sum3 + Math.log(this.sum3 / this.twopi) / Math.log(10.);
        out.ajouterString("comment", "chi^2 marginalisé sur H0 = " +
            (Math.floor(10. * this.chi2) / 10.) + " pour " + this.len + " données");
    }
    
    return out;
}


DarkEnergy.prototype.rombint = function(func, x, y, tol) {
    //  Rombint returns the integral from a to b of f(x)dx using Romberg integration.
    //  The method converges provided that f(x) is continuous in (a,b).  
    //  The function f must be double precision and must be declared external 
    //  in the calling routine.  
    //  tol indicates the desired relative accuracy in the integral.
    
    var step, gmax, error, g0, fourj, g1, nint, i, jmax, maxiter, maxj, g, k, j;
    maxiter = 20;
    maxj = 5;
    g = new Array(maxj + 2);
    
    step = 0.5 * (y - x);
    gmax = step * (func.apply(this, [y]) + func.apply(this, [x]));
    g[1] = gmax;
    nint = 1;
    error = 1.e20;
    i = 0;
    g0 = 0;
    jmax = 0;
    while (!(i > maxiter || (i > 5 && Math.abs(error) < tol))) {
        i++;
        //	Calculate next trapezoidal rule approximation to integral.
        g0 = 0;
        for (k=0; k<nint; k++) {
            g0 = g0 + func.apply(this, [x + (k + k - 1) * step]);
        }
        g0 = 0.5 * g[1] + step * g0;
        step = 0.5 * step;
        nint = nint + nint;
        jmax = Math.min(i, maxj);
        fourj = 1;
        for (j=0; j<jmax; j++) {
            //	Use Richardson extrapolation.
            fourj = 4. * fourj;
            g1 = g0 + (g0 - g[j]) / (fourj - 1.);
            g[j] = g0;
            g0 = g1;
        }
        if (Math.abs(g0) > tol) {
            error = 1. - gmax / g0;
        } else {
            error = gmax;
        }
        gmax = g0;
        g[jmax+1] = g0;
    }
    return(g0);
}

DarkEnergy.prototype.trim = function(s) {
    return(s.replace(/^\s+|\s+$/g, ""));
}

/**
* lire un fichier ASCII 
* avec le nombre d'éléments sur la première ligne
* et " x  y  z" sur les suivantes -> t[0][i] = x; t[1][i] = y; t[2][i] = z;
*/
DarkEnergy.prototype.lireFichier = function(nomFichier) {
    var xmlhttp, texte, ligne, t, n, i, is, sy, s, y, sz, z;
    
    xmlhttp = new XMLHttpRequest();
    if (xmlhttp.overrideMimeType)
        xmlhttp.overrideMimeType('text/plain');
    try {
        xmlhttp.open("GET", nomFichier, false);
        xmlhttp.send();
        lignes = xmlhttp.responseText.split("\n");
        ligne = lignes[0];
        n = parseInt(this.trim(ligne));
        t = new Array(3);
        for (i=0; i<3; i++)
            t[i] = new Array(n);
        for (i=0; i < n && ligne != null; i++) {
            ligne = lignes[i + 1];
            if (ligne != null) {
                ligne = this.trim(ligne);
                is = ligne.indexOf(' ');
                sx = this.trim(ligne.substring(0,is));
                ligne = this.trim(ligne.substring(is+1));
                is = ligne.indexOf(' ');
                sy = this.trim(ligne.substring(0,is));
                x = parseFloat(sx);
                y = parseFloat(sy);    
                sz = this.trim(ligne.substring(is+1));
                z = parseFloat(sz);		
                t[0][i] = x;
                t[1][i] = y;
                t[2][i] = z;
            }
        }

        return(t);
        
    } catch (ex) {
        alert("Erreur à la lecture du fichier de données: " + ex);
        return(null);
    }
}

/* inutilisé
DarkEnergy.prototype.f1 = new function(x, f) { // fonction à intégrer pour avoir le temps (1/(a*H))
    f = this.Omega_m0 * Math.pow(this.a0/x, 3.)
        + this.Omega_rad0 * Math.pow(this.a0/x, 4.)
        - this.Omega_k0 * Math.pow(this.a0/x, 2.)
        + this.Omega_l0;
    f = 1 / (this.h0 * Math.sqrt(f)) / x * this.gyr;
    return(f);
};
*/

DarkEnergy.prototype.f2 = function(x) { // fonction à intégrer pour avoir la distance lumineuse (H0/H)
    var f;
    f = this.Omega_m0 * Math.pow((1.+x), 3.)
        + this.Omega_rad0 * Math.pow((1.+x), 4.)
        - this.Omega_k0 * Math.pow((1.+x), 2.)
        + this.Omega_l0;
    f = 1 / Math.sqrt(f);
    return(f);
}
